package lc.world.objectes;

import com.badlogic.gdx.graphics.g2d.Sprite;
import com.badlogic.gdx.graphics.g2d.SpriteBatch;
import com.badlogic.gdx.math.Rectangle;
import java.nio.ByteBuffer;
import lc.item.block.Block;
import lc.item.block.TypeBlock;
import lc.world.Ground;
import lc.world.World;

/**
 * @author Death
 */
public class WorldObject{

    public static final int UP = 0, DOWN = 1, LEFT = 2, RIGHT = 3, VERTICALLY = 4, HORIZONTAL = 5;
    
    
    protected World world = null;
    protected Ground ground = null;
    protected final Rectangle rect;
    protected boolean accessoryFlag1; // Вспомогательные флаги
    
    public CollisionGround collisionGround = null;
    public CollisionGroundExpose groundExpose = null;
    public boolean onLand, freeSide;
    
    public int dirX, dirY;
    public float x, y, speedX, speedY;
    private float deltaX, deltaY;
    public float maxSpeedX, maxSpeedY;
    public float width, height;
    public float heightJump;
    public float beginHeightJump;
    public float acceleration; // Ускорение
    public double powAcceleration; // Степень сукорения
    public double strengthJump; // Сила прыжка
    public double strengthGravity; // Сила Гравитации
    public double maxSpeedGravity; // Максемальная скорость Гравитации
    public boolean jump;
    public boolean gravity;
    public boolean autoBrakeX, autoBrakeY;
    public boolean[] moveKey;
    public boolean[] moveDir;
    public Sprite sprite = null;
    public TypeObjectWorld type;
    
    public int group = 0; // Для игры, группы объектов
    
    private float beginFall = 0;
    
    public WorldObject(){
        this.rect = new Rectangle();
        this.type = TypeObjectWorld.STATIC;
        this.moveKey = new boolean[4];
        this.moveDir = new boolean[4];
        this.maxSpeedX = 8;
        this.maxSpeedY = 8;
        this.gravity = false;
        this.onLand = false;
        this.freeSide = true;
        this.acceleration = 0.25f;
        this.powAcceleration = 1.1;
        this.strengthJump = 6.5;
        this.strengthGravity = 0.25f;
        this.maxSpeedGravity = this.maxSpeedY * 2;
        this.beginHeightJump = 0;
        this.heightJump = Block.SIZE * 4.5f;
        this.jump = false;
        this.autoBrakeX = false;
        this.autoBrakeY = false;
        
        this.accessoryFlag1 = false;
    }
    
    public void draw(SpriteBatch b){
        if(sprite == null) return;
        sprite.setPosition(x, y);
        sprite.setSize(width, height);
        sprite.draw(b);
    }
    
    public Rectangle getRect(){
        return rect.set(x, y, width, height);
    }
    
    public void initPhysics(World map){
        this.initPhysics(map, true);
    }
    
    public void initPhysics(World map, boolean g){
        this.gravity = g;
        this.world = map;
        this.ground = map.ground;
    }
    
    public void move(){
        deltaX = x;
        deltaY = y;
        if(moveKey[RIGHT]){
            speedX += this.voile();
            moveDir[RIGHT] = true;
            if(speedX > maxSpeedX) speedX = maxSpeedX;
        }else{
            if(moveDir[RIGHT]){
                speedX -= this.brake();
                if(speedX < 0){
                    speedX = 0;
                    moveDir[RIGHT] = false;
                }
            }
        }
        
        if(moveKey[LEFT]){
            speedX -= this.voile();
            moveDir[LEFT] = true;
            if(speedX < -maxSpeedX) speedX = -maxSpeedX;
        }else{
            if(moveDir[LEFT]){
                speedX += this.brake();
                if(speedX > 0){
                    speedX = 0;
                    moveDir[LEFT] = false;
                }
            }
        }
        
        if(autoBrakeX){
            if(speedX < 0.0){
                if(onLand)
                    speedX += this.voile(20);
                else
                    speedX += this.voile(5);
                if(speedX > 0.0) speedX = 0;
            }else if(speedX > 0.0){
                if(onLand)
                    speedX -= this.voile(20);
                else
                    speedX -= this.voile(5);
                if(speedX < 0.0) speedX = 0;
            }
        }
        
        this.gravity();
        
        this.moveY(speedY);
        this.moveX(speedX);
        
        if(onLand) this.moveY(-1); // Проверка на землю, если на земле =).
    }
    
    public float getDeltaX(){
        return deltaX - x;
    }
    
    public float getDeltaY(){
        return deltaY - y;
    }
    
    private void gravity(){
        if(jump){
            if(speedY <= 0){
                jump = false;
                return;
            }
            if(this.y > this.beginHeightJump + heightJump){
                speedY -= (strengthGravity * 2);
            }else{
                speedY -= (strengthGravity / 4);
            }
            return;
        }else{
            if(speedY > 0){
                if(this.y > this.beginHeightJump + Block.SIZE * 2){
                    speedY -= (strengthGravity * 3);
                    return;
                }else{
                    speedY -= (strengthGravity / 4);
                }
            }
        }
        if(!this.gravity || onLand) return;
        if(beginFall == 0){
            if(speedY < 0)
                beginFall = y;
        }
        
        speedY -= strengthGravity;
        if(speedY > maxSpeedGravity) speedY = (int)-maxSpeedGravity;
    }
    
    public void jump(boolean flag){
        jump = flag;
        if(!onLand) return;// Если не земля то не даём прыгнуть
        if(jump){
            beginHeightJump = this.y;
            speedY = (float)strengthJump;
        }
    }
    
    public boolean moveX(float speed){
        if(speed == 0) return false;
        if(speed > 0) dirX = RIGHT;
        else dirX = LEFT;
        if(Math.abs(speed) > Block.SIZE)
            return this.superMoveX(Math.round(speed));
        this.x = this.x + Math.round(speed);
        return collisionGround(speed, HORIZONTAL);
    }
    public boolean moveY(float speed){
        if(speed == 0) return false;
        if(speed > 0) dirY = UP;
        else dirY = DOWN;
        if(Math.abs(speed) > Block.SIZE)
            return this.superMoveY(Math.round(speed));
        this.y = this.y + Math.round(speed);
        return collisionGround(speed, VERTICALLY);
    }
    
    private boolean superMoveX(int speed){
        int sizeCell = this.getSizeBlock();
        int sx = Math.abs(speed);
        int vx = sx / speed;
        while(sx > sizeCell){
            this.x = this.x + Block.SIZE * vx;
            sx -= sizeCell;
            accessoryFlag1 = collisionGround(speed, HORIZONTAL);
            if(speed == 0){
                sx = 0;
                break;
            }
            if(accessoryFlag1){
                return true;
            }
        }
        this.x = this.x + sx * vx;
        return collisionGround(speed, HORIZONTAL);
    }
    
    private boolean superMoveY(int speed){
        int sizeCell = this.getSizeBlock();
        int sy = Math.abs(speed);
        int vy = sy / speed;
        while(sy > sizeCell){
            this.y = this.y + sizeCell * vy;
            sy -= sizeCell;
            accessoryFlag1 = collisionGround(speed, VERTICALLY);
            if(speed == 0){
                sy = 0;
                break;
            }
            if(accessoryFlag1){
                return true;
            }
        }
        this.y = this.y + sy * vy;
        return collisionGround(speed, VERTICALLY);
    }
    
    public boolean collisionGround(float speed, int dir){
        if(world == null || ground == null) return false;
        if(collisionBorder()) return true;
        for(int i = (int)(this.x / this.getSizeBlock()); i < Math.floor(this.x + this.width) / this.getSizeBlock(); i++){
            for(int j = (int)(this.y / this.getSizeBlock()); j < Math.floor(this.y + this.height) / this.getSizeBlock(); j++){
                boolean collision = ground.getBlock(i, j) != null;
                if(collision)
                    if(ground.getBlock(i, j).type == TypeBlock.PASSABLE) continue;
                if(collision && groundExpose != null)
                    collision = groundExpose.expose(ground.getBlock(i, j), i, j);
                if(collision){
                    int dirColl = -1;
                    if(dir == HORIZONTAL){
                        if(speed > 0){
                            this.x = i * this.getSizeBlock() - this.width;
                            dirColl = RIGHT;
                            moveDir[RIGHT] = false;
                        }else if(speed < 0){
                            this.x = i * this.getSizeBlock() + this.getSizeBlock();
                            dirColl = LEFT;
                            moveDir[LEFT] = false;
                        }
                        speedX = 0;
                    }else if(dir == VERTICALLY){
                        if(speed > 0){
                            this.y = j * this.getSizeBlock() - this.height;
                            dirColl = UP;
                            moveDir[UP] = false;
                        }else if(speed < 0){
                            if(beginFall != 0) this.fall(beginFall - this.y);
                            beginFall = 0;
                            this.onLand = true;
                            this.y = j * this.getSizeBlock() + this.getSizeBlock();
                            dirColl = DOWN;
                            moveDir[DOWN] = false;
                        }
                        speedY = 0;
                    }
                    if(dirColl != -1 && collisionGround != null){
                        collisionGround.collision(i, j, dirColl);
                    }
                    return true;
                }
            }
        }
        if(dir == VERTICALLY){
            this.onLand = false;
        }
        return false;
    }
    
    public boolean isCollisionBlock(int dx, int dy){
        int xID = (int)Math.floor((this.x + dx) / Block.SIZE);
        int yID = (int)Math.floor((this.y + dy) / Block.SIZE);
        if(ground.getBlock(xID, yID) != null){
            return ground.getBlock(xID, yID).type != TypeBlock.PASSABLE;
        }
        return false;
    }
    public void fall(float heightFall){}
    
    private boolean collisionBorder(){
        if(this.x < 0){
            this.x = 0;
            return true;
        }
        if(this.y < 0){
            this.y = 0;
            this.onLand = true;
            return true;
        }
        if(this.x + this.width > world.getWidth()){
            this.x = world.getWidth() - this.width;
            return true;
        }
        if(this.y + this.height > world.getHeight()){
            this.y = world.getHeight() - this.height;
            return true;
        }
        return false;
    }
    
    public float getCenterXId(){
        return (x + width / 2) / Block.SIZE;
    }
    
    public float getCenterYId(){
        return (y + height / 2) / Block.SIZE;
    }
    
    public int getSizeBlock(){
        return Block.SIZE;
    }
    
    private float voile(){// Ускорение в бока
        return voile(acceleration);
    }
    private float voile(float acc){// Общее скорение
        return (float)Math.pow(acc, this.powAcceleration);
    }
    private float brake(){// Тормажение в бока
        return brake(acceleration);
    }
    private float brake(float acc){// Общее тормажение
        return (float)Math.pow(acc * 2, this.powAcceleration);
    }
    
    public byte[] getData(){
        byte[] data = ByteBuffer.allocate(100)
                .putInt((int)x)
                .putInt((int)y)
                .putInt((int)width)
                .putInt((int)height)
                .putFloat(heightJump)
                .putFloat(maxSpeedX)
                .putFloat(maxSpeedY)
                .array();
        return data;
    }
    
    public void setData(byte[] data){
        ByteBuffer buf = ByteBuffer.wrap(data);
        x = buf.getInt();
        y = buf.getInt();
        width = buf.getInt();
        height = buf.getInt();
        heightJump = buf.getFloat();
        maxSpeedX = buf.getFloat();
        maxSpeedY = buf.getFloat();
    }
}
